/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.io.File;
import java.util.Map;

import org.objectweb.fractal.adl.util.Printer;

/**
 * GCC compiler wrapper.
 */
public class Gcc extends AbstractBuilder implements CompilerBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the CompilerBuilder interface
  // --------------------------------------------------------------------------

  public void compile(final File srcFile, final File destFile,
      final String cflags, final Map<Object, Object> context) throws Exception {

    if (!destFile.getParentFile().exists()) destFile.getParentFile().mkdirs();

    final CommandLine cmd = new CommandLine();
    cmd.setExecutable(getContextArg("compiler-command", context));

    // Creating attribute compiler arguments
    if (getFlags() != null) cmd.createArgument(getFlags());

    // Creating component specific compiler arguments
    if (cflags != null) cmd.createArgument(cflags);

    // Creating default compiler arguments
    final String ctxCFlags = tryGetContextArg("c-flags", context);
    final String ctxDefs = tryGetContextArg("defs", context);
    final String ctxIncPath = tryGetContextArg("inc-path", context);
    if (ctxCFlags != null) cmd.createArgument(ctxCFlags);
    if (ctxDefs != null) cmd.createArgument(ctxDefs);
    if (ctxIncPath != null) cmd.createArgument(ctxIncPath);

    cmd.createArgument("-o " + destFile.getPath());
    cmd.createArgument(srcFile.getPath());

    if (Printer.isDebugEnable())
      Printer.debug("[gcc] " + cmd.getCommandLine());
    else if (Printer.isInfoEnable())
      Printer.info("[gcc] " + destFile.getPath());

    final Process child = Runtime.getRuntime().exec(cmd.getCommandLine());
    // Nothing will be printed if there is no errors nor warnings
    printCompilerMessages(child);
    // Catch the return value, throw an exception if there is an error
    if (child.waitFor() != 0)
      throw new CompilationException("Error while compiling the file \""
          + srcFile.getAbsolutePath() + "\"");
    child.destroy();
  }
}
