/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.task.codegen.lib.AggregateFileCollectionProviderTask;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Adds output task which provides a collection of every compiled files used by
 * the compiled component.
 */
public class ObjOutputCompiler extends AbstractOutputCompiler {

  // --------------------------------------------------------------------------
  // Implementation of the PrimitiveCompiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {

    try {
      tasks.getTask("output", container);
      return;
    } catch (NoSuchElementException e) {
      // continue
    }

    final AggregateFileCollectionProviderTask outputTask = new AggregateFileCollectionProviderTask();
    tasks.addTask("output", container, outputTask);

    final Collection<TaskMap.TaskHole> compilationTasks = getComponentCompilationTasks(
        container, tasks, context);
    for (TaskMap.TaskHole taskHole : compilationTasks) {
      outputTask.addFileProviderTask(taskHole);
    }
    addSubCompDependancies(container, tasks, outputTask);
  }
}
