/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.interfaces;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.interfaces.Interface;
import org.objectweb.fractal.adl.interfaces.InterfaceContainer;
import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.task.codegen.api.RequireSourceCodeProviderTask;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Adds a task for all the interfaces.
 */
public class InterfaceCompiler extends AbstractInterfaceCompiler {

  // --------------------------------------------------------------------------
  // Implementation of the Compiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {
    final Interface[] itfs = ((InterfaceContainer) container).getInterfaces();
    if (itfs.length == 0) return;
    final TypeInterface[] interfaces = new TypeInterface[itfs.length];
    System.arraycopy(itfs, 0, interfaces, 0, itfs.length);

    createTask(container, tasks, interfaces);
  }

  protected void createTask(final ComponentContainer container,
      final TaskMap tasks, final TypeInterface[] interfaces) {
    final TaskMap.TaskHole taskHole = createTask("interfaces", interfaces,
        container, tasks);
    getTaskHole("component", container, tasks).addDependency(taskHole,
        RequireSourceCodeProviderTask.SOURCE_CODE_PROVIDER_TASK_ROLE,
        "interfaces");
  }
}
