/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.components;

import java.util.Map;

import org.objectweb.fractal.cecilia.adl.components.ComponentBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the instantiation of a primitive component written in
 * <code>thinkMC</code> languages.
 */
public class ComponentInstanceBuilder implements ComponentBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the ComponentBuilder interface
  // --------------------------------------------------------------------------

  public Object createComponent(final Object componentType,
      final Object componentInstance, final Object controllerDesc,
      final Object contentDesc, final Map<String, Object> codePieces,
      final Map<Object, Object> context) throws Exception {
    final CodeWriter cw = new CodeWriter("Primitive Instantiation Builder");
    final String definitionName = componentType.toString().replace('.', '_');
    final String instanceName = componentInstance.toString().replace('.', '_');

    cw.append("/* Create instance ").append(componentInstance.toString())
        .append(" of component type ").append(componentType.toString()).append(
            "*/").endl();

    cw.appendln((String) codePieces.get("exported-interfaces-vft-declaration"));

    // Declare the component structure since it may be referenced by the
    // controller structures.
    // This declaration is extern since it is redeclared and initialized latter.
    // This avoid warnings when compiling with -Wredundant-decls.
    cw.append("#ifndef ").append(instanceName).append("_DECLARED").endl();
    cw.append("extern struct ").append(definitionName).append("_t ").append(
        instanceName).append(";").endl();
    cw.append("#define ").append(instanceName).append("_DECLARED").endl();
    cw.appendln("#endif");

    cw.appendln((String) codePieces.get("controller"));

    cw.appendln((String) codePieces.get("interfaces.include"));
    // Create a new instance of this component
    cw.append("struct ").append(definitionName).append("_t ").append(
        instanceName).append(" = {").endl();
    /* Initialize this component data structure */
    cw.append((String) codePieces.get("interfaces.init")).append(",").endl();

    final String attrInitVals = (String) codePieces.get("attributes");
    if (attrInitVals != null) cw.append(attrInitVals).append(",").endl();
    cw.append("};").endl();

    return cw.toString();
  }
}
