/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 * Contributor: Alessio Pace (const attributes)
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.controllers;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.attributes.Attribute;
import org.objectweb.fractal.cecilia.adl.attributes.AttributeBuilder;
import org.objectweb.fractal.cecilia.adl.idl.ast.Field;
import org.objectweb.fractal.cecilia.adl.idl.ast.FieldContainer;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Build the definition for the internal data of the attribute controller.
 */
public class ACDefinitionBuilder implements AttributeBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the AttributesBuilder interface
  // --------------------------------------------------------------------------

  public Object setAttributes(final Object componentType,
      final Object componentInstance, final Object attributeAST,
      final Attribute[] fields, final Map<String, Object> codePieces,
      final Map<Object, Object> context) {
    final CodeWriter cw = new CodeWriter(
        "Attribute Controller Definition Builder");

    final Field[] recordFields = ((FieldContainer) attributeAST).getFields();

    /*
     * it will contain a list of the fields (attributes) names with the "const"
     * qualifier
     */
    final List<String> constantFieldNames = new ArrayList<String>();

    final String escapedComponentTypeName = componentType.toString().replace(
        '.', '_');
    cw.append("struct ").append(escapedComponentTypeName).append(
        "_attributedskeleton").append(" {").endl();
    cw.append("  unsigned int  nbAttribute;").endl();
    cw.append(" struct {").endl();

    for (final Field f : recordFields) {

      final String fieldName = f.getName();
      cw.append(" const char *attribute_" + fieldName + ";").endl();

      /*
       * if the Field is declared as const in the IDL, then the value pointed by
       * this pointer should be const
       */
      if (f.getQualifier().equals(Field.CONST)) {
        cw.append(" const");
        /* add this field name to the list of const field names */
        constantFieldNames.add(fieldName);
      }
      cw.append(" void *value_" + fieldName + ";").endl();
      cw.append(" char size_" + fieldName + ";").endl();
    }

    cw.append("}attributes[1];").endl();

    /*
     * the following are informations for the AttributeController about which
     * fields are constants, so that write access on them would raise an error
     * code result.
     */
    cw.append(" unsigned int nbConstAttribute;").endl();
    cw.append(
        " const char* const_attribute_names[" + constantFieldNames.size()
            + "];").endl();
    cw.append("};").endl();

    cw.endl();

    return cw.toString();
  }
}
