/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.controllers;

import java.util.Map;

import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * ComponentIdentity internal data definition builder.
 */
public class CIDefinitionBuilder implements InterfaceBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the InterfaceBuilder interface
  // --------------------------------------------------------------------------

  public Object createInterfaces(final Object componentType,
      final Object componentInstance, final Object contentType,
      final TypeInterface[] interfaceTypes,
      final Map<String, Object> codePieces, final Map<Object, Object> context) {
    final CodeWriter cw = new CodeWriter(
        "Component Identity Definition Builder");

    cw.append("struct ").append(componentType.toString().replace('.', '_'))
        .append("_CI_data_type").append("{").endl();
    cw.append("  unsigned int nbexported;").endl();
    cw.append("  struct {").endl();
    cw.append("    const char *name;").endl();
    cw.append("    void *itf;").endl();
    cw.append("  } exported[").append(interfaceTypes.length).append("];")
        .endl();
    cw.append("} ;").endl();

    return cw.toString();
  }

}
