/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.source;

import static org.objectweb.fractal.cecilia.primitive.thinkMC.source.DefinitionSourceFileBuilder.getDefinitionSourceFileName;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.ContextLocal;
import org.objectweb.fractal.cecilia.adl.source.AbstractSourceFileBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;
import org.objectweb.fractal.task.codegen.util.SourceFile;

/**
 * Creates C source file that includes the C-header file defining the component
 * structure and an implementation module. This builder is applicable for
 * multi-module primitive component written in <code>ThinkMC</code>.
 */
public class ModuleSourceFileBuilder extends AbstractSourceFileBuilder {

  ContextLocal<Map<Object, Integer>> moduleCounters = new ContextLocal<Map<Object, Integer>>();

  // The signature of the file that is being built.
  ThreadLocal<String>                curSignature   = new ThreadLocal<String>();

  @Override
  protected String getFileName(final Object componentType,
      final Object componentInstance, final Map<Object, Object> context) {
    Map<Object, Integer> moduleCounter = moduleCounters.get(context);
    if (moduleCounter == null) {
      moduleCounter = new HashMap<Object, Integer>();
      moduleCounters.set(context, moduleCounter);
    }
    Integer counter = moduleCounter.get(componentType);
    if (counter == null) {
      counter = 1;
    }
    moduleCounter.put(componentType, counter + 1);

    final String fileName = componentType.toString().replace('.',
        File.separatorChar)
        + ".adl-module" + counter + ".c";
    final String signature = componentType.toString() + "-module" + counter;
    // set signature as current signature in thread-local variable.
    curSignature.set(signature);

    return fileName;
  }

  @Override
  protected String getFileContent(final Object componentType,
      final Object componentInstance, final Map<String, Object> codePieces,
      final Map<Object, Object> context) {
    final CodeWriter cw = new CodeWriter();

    cw.append("#include \"").append(getDefinitionSourceFileName(componentType))
        .append("\"").endl();

    final String module = (String) codePieces.get("module");
    if (module == null)
      throw new IllegalArgumentException(
          "Cannot find the \"module\" code piece.");
    cw.appendln(module);
    return cw.toString();
  }

  @Override
  protected SourceFile getSourceFile(final Object componentType,
      final Object componentInstance, final File file) {
    return new SourceFile(curSignature.get(), file);
  }
}
